#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  now.pm - Show songs currently playing
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# Version:
#  $Id: now.pm,v 1.8 2003/04/11 14:04:03 stevekemp Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/
#
#


use gnump3d::files;

#
#  Return the author of this plugin.
#
sub getAuthor()
{
    return( 'Steve Kemp <steve@steve.org.uk>' );
}


#
#  Return the version of this plugin.
#
sub getVersion()
{
    my $REVISION      = '$Id: now.pm,v 1.8 2003/04/11 14:04:03 stevekemp Exp $';
    my $VERSION       = "";
    $VERSION = join (' ', (split (' ', $REVISION))[1..3]);
    $VERSION =~ s/,v\b//;
    $VERSION =~ s/(\S+)$/($1)/;

    return( $VERSION );
}



#
#  Handle requests to this plugin.
#
sub handlePath( )
{
    my ( $uri ) = (@_);

    my $header   = &getHTTPHeader( 200, "text/html" );
    &sendData($data, $header );

    #
    # TODO: Make this code work under windows.
    #       The filedb thing isn't supported..
    #
    if ( &isWindows() )
    {
      print "Windows server ignoring /now/\n";
      return( &windowsError( ) );
      exit; # NOP
    }


    my $NOW_PLAYING_FILE  = '/tmp/now_playing.db';
    my %NOW_PLAYING       = ();

    tie( %NOW_PLAYING, 'DB_File', $NOW_PLAYING_FILE );
    my $output = "";
    $output .= "<table>\n";

    my $count = 0;

    foreach my $key ( sort keys( %NOW_PLAYING ) )
    {
        my $file    = $NOW_PLAYING{$key};
	my $display = "";   # Tag info to display.
	my $link    = "";   # Link to the file.
	my $dir     = "";   # Link to the containing directory.

	# We have one more track being played.
	$count += 1;

	# Get the tag info.
	if ( ( -e $file ) && ( -f $file ) )
	{
	    $display = &getSongDisplay( $file, $song_format );
	}
	else
	{
	    $display   = $file;
	    if ( $display =~ /(.*)\.(.*)/ )
	    {
	        $display = $1;
	    }
	    if ( $display =~ /(.*)\/(.*)/ )
	    {
	        $display = $2;
	    }
	}
	$link   = substr( $file, length( $ROOT ) );

	if ( -d $file )
	{
	  $dir = $link;
	}
	else
	{
	  if ( $link =~ /(.*)\/(.*)/ )
	    {
	      $dir = $1;
	    }
	  else
	    {
	      $dir = "/";
	    }
	}

	$key = &ipToName( $key );

	$output .= "<tr><td><b>$key</b></td><td><a href='$link'>$display</a></td><td><a href='$dir/'>$dir</a></td></td>\n";
    }

    if ( $count eq 0 )
    {
        $output .= "<tr><td>There are no songs currently being served.</td></tr>\n";
    }
    $output .= "</table>\n";



    my $text = createOutput( $output );
    &sendData($data, $text );
    close( $data );
    return 1;
}

sub windowsError( ) 
{
  my $text = "<tr><td><p>&nbsp;This plugin currently doesnt work under windows.</p><p>&nbsp;Please do not report this as a bug.</p></td></tr>\n";

  my $out = &createOutput( $text );

  sendData( $data, $out );
  close( $data );
  return 1;
}



sub createOutput( $ )
{
  my ( $output ) = ( @_ );

  my @template = &getThemeFile( $ARGUMENTS{'theme'}, "plugin.html" );
  my $text = "";

  foreach my $line (@template )
  {
      #
      # Make global substitutions.
      #
      $line =~ s/\$HEADER/<META HTTP-EQUIV=\"refresh\" content=\"30;\">/g;
      $line =~ s/\$HOSTNAME/$host/g;
      $line =~ s/\$VERSION/$VERSION/g;
      $line =~ s/\$DIRECTORY/\/bug\//g;
      $line =~ s/\$HEADING/Now Playing/g;
      $line =~ s/\$TITLE/Now Playing/g;

      #
      # Now handle the special sections.
      #
      if ( $line =~ /(.*)\$BANNER(.*)/ )
      {
	  # Insert banner;
	  my $pre  = $1;
	  my $post = $2;

          $text .= $pre;
	  $text .= &getBanner( "/now/" );
	  $text .= $post;
      }
      elsif ( $line =~ /(.*)\$TEXT(.*)/ )
      {
	  $text .= $1 . $output . $2;
      }
      else
      {
	  $text .= $line;
      }
   }

   return( $text );
}



#
#  Convert a given IP address to a hostname.
#
sub ipToName($)
{
    my( $ip ) = ( @_ );

    my @address = gethostbyaddr( pack( 'C4', split(/\./, $ip)), 2 );
    return(  @address > 0 ? $address[0] : $ip );
}
